@@ -1,5 +1,5 @@
 /*
- *	Copyright © Jan Engelhardt, 2008 - 2010
+ *	Copyright © Jan Engelhardt, 2008-2011
  *
  *	This file is part of pam_mount; you can redistribute it and/or
  *	modify it under the terms of the GNU Lesser General Public License
@@ -21,15 +21,18 @@
 #include <string.h>
 #include <unistd.h>
 #include <libHX.h>
+#include "libcryptmount.h"
 #include "pam_mount.h"
 #ifdef HAVE_LIBCRYPTO
 #	include <openssl/evp.h>
 #endif
+#include "cmt-internal.h"
 
 /**
  * @object:		volume or mountpoint; used by remount
  * @container:		path to the volume (like (bdev) /dev/sda2 or
  * 			(file) /home/user.!@#)
+ * @crypto_name:	preferred name of the crypto device (where possible)
  * @mountpoint:		where to put this
  * @extra_options:	options to pass down
  * @no_update:		do not update mtab
@@ -43,8 +46,9 @@
  * @remount:		issue a remount
  */
 struct mount_options {
-	const char *object, *container, *mountpoint, *fstype;
-	const char *dmcrypt_cipher, *dmcrypt_hash;
+	hxmc_t *object, *container, *mountpoint;
+	const char *fstype;
+	const char *crypto_name, *dmcrypt_cipher, *dmcrypt_hash;
 	const char *fsk_hash, *fsk_cipher, *fsk_file;
 	hxmc_t *fsk_password, *extra_opts, *crypto_device;
 	char *loop_device;
@@ -64,56 +68,12 @@ struct mount_options {
  * @blkdev:		@container is a block device
  */
 struct umount_options {
-	const char *object;
+	hxmc_t *object;
 	unsigned int no_update, ro_fallback;
 	bool is_cont, blkdev;
 };
 
-static const char *const mtab_file  = "/etc/mtab";
-static const char *const kmtab_file = "/proc/mounts";
-
-static hxmc_t *readlinkf(const char *file)
-{
-	static const unsigned int bufsize = 256;
-	const char *const args[] = {"readlink", "-fn", file, NULL};
-	struct HXproc proc;
-	ssize_t readret;
-	char *buf;
-	int ret;
-
-	if ((buf = malloc(bufsize)) == NULL) {
-		fprintf(stderr, "malloc: %s\n", strerror(errno));
-		return NULL;
-	}
-	arglist_llog(args);
-	memset(&proc, 0, sizeof(proc));
-	proc.p_flags = HXPROC_VERBOSE | HXPROC_STDOUT;
-	if ((ret = HXproc_run_async(args, &proc)) <= 0) {
-		fprintf(stderr, "spawn_startl readlink: %s\n", strerror(-ret));
-		free(buf);
-		return NULL;
-	}
-
-	readret = read(proc.p_stdout, buf, bufsize);
-	if (readret < 0) {
-		fprintf(stderr, "read: %s\n", strerror(errno));
-		goto out;
-	} else if (readret >= bufsize) {
-		fprintf(stderr, "Hm, path too long for us\n");
-		goto out;
-	}
-
-	buf[readret] = '\0';
-	close(proc.p_stdout);
-	HXproc_wait(&proc);
-	return buf;
-
- out:
-	free(buf);
-	close(proc.p_stdout);
-	HXproc_wait(&proc);
-	return NULL;
-}
+static unsigned int mtcr_debug;
 
 static void mtcr_parse_suboptions(const struct HXoptcb *cbi)
 {
@@ -122,6 +82,7 @@ static void mtcr_parse_suboptions(const struct HXoptcb *cbi)
 	bool first = true;
 	char *copt;
 	char *key;
+	int ret;
 
 	if ((copt = xstrdup(cbi->data)) == NULL)
 		return;
@@ -137,17 +98,26 @@ static void mtcr_parse_suboptions(const struct HXoptcb *cbi)
 			++key;
 		if (strcmp(key, "cipher") == 0) {
 			mo->dmcrypt_cipher = value;
-			if (cipher_digest_security(value) < 1)
+			ret = ehd_cipherdigest_security(value);
+			if (ret < 0)
+				fprintf(stderr, "%s\n", strerror(-ret));
+			else if (ret < EHD_SECURITY_UNSPEC)
 				fprintf(stderr, "Cipher \"%s\" is considered "
 				        "insecure.\n", value);
 		} else if (strcmp(key, "fsk_cipher") == 0) {
 			mo->fsk_cipher = value;
-			if (cipher_digest_security(value) < 1)
+			ret = ehd_cipherdigest_security(value);
+			if (ret < 0)
+				fprintf(stderr, "%s\n", strerror(-ret));
+			else if (ret < EHD_SECURITY_UNSPEC)
 				fprintf(stderr, "Cipher \"%s\" is considered "
 				        "insecure.\n", value);
 		} else if (strcmp(key, "fsk_hash") == 0) {
 			mo->fsk_hash = value;
-			if (cipher_digest_security(value) < 1)
+			ret = ehd_cipherdigest_security(value);
+			if (ret < 0)
+				fprintf(stderr, "%s\n", strerror(-ret));
+			else if (ret < EHD_SECURITY_UNSPEC)
 				fprintf(stderr, "Hash \"%s\" is considered "
 				        "insecure.\n", value);
 		} else if (strcmp(key, "dm-timeout") == 0)
@@ -165,12 +135,19 @@ static void mtcr_parse_suboptions(const struct HXoptcb *cbi)
 			l0g("loop mount option ignored\n");
 		else if (strcmp(key, "hash") == 0) {
 			mo->dmcrypt_hash = value;
-			if (cipher_digest_security(value) < 1)
+			ret = ehd_cipherdigest_security(value);
+			if (ret < 0)
+				fprintf(stderr, "%s\n", strerror(-ret));
+			else if (ret < EHD_SECURITY_UNSPEC)
 				fprintf(stderr, "Hash \"%s\" is considered "
 				        "insecure.\n", value);
-		} else if (strcmp(key, "verbose") == 0)
-			Debug = pmtlog_path[PMTLOG_DBG][PMTLOG_STDERR] = true;
-		else {
+		} else if (strcmp(key, "verbose") == 0) {
+			if (!mtcr_debug)
+				ehd_logctl(EHD_LOGFT_DEBUG, EHD_LOG_SET);
+			mtcr_debug = true;
+		} else if (strcmp(key, "crypto_name") == 0) {
+			mo->crypto_name = value;
+		} else {
 			if (!first)
 				HXmc_strcat(&passthru, ",");
 			first = false;
@@ -185,9 +162,9 @@ static void mtcr_parse_suboptions(const struct HXoptcb *cbi)
 		if (strcmp(key, "remount") == 0)
 			mo->remount = true;
 		else if (strcmp(key, "ro") == 0)
-			mo->readonly = true;
+			mo->readonly = EHD_LOSETUP_RO;
 		else if (strcmp(key, "rw") == 0)
-			mo->readonly = false;
+			mo->readonly = EHD_LOSETUP_RW;
 	}
 
 	if (*passthru != '\0') {
@@ -222,7 +199,7 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 		 .uptr = opt, .help = "Mount options"},
 		{.sh = 'r', .type = HXTYPE_NONE, .ptr = &opt->readonly,
 		 .help = "Set up devices and mounts as read-only"},
-		{.sh = 'v', .type = HXTYPE_NONE, .ptr = &Debug,
+		{.sh = 'v', .type = HXTYPE_NONE, .ptr = &mtcr_debug,
 		 .help = "Enable debugging"},
 		HXOPT_AUTOHELP,
 		HXOPT_TABLEEND,
@@ -230,10 +207,12 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 	bool kfpt;
 	int ret;
 
-	if (HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR) <= 0)
+	if (HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR) !=
+	    HXOPT_ERR_SUCCESS)
 		return false;
 
-	pmtlog_path[PMTLOG_DBG][PMTLOG_STDERR] = Debug;
+	if (mtcr_debug)
+		ehd_logctl(EHD_LOGFT_DEBUG, EHD_LOG_SET);
 
 	if (opt->remount) {
 		if (*argc < 2 || *(*argv)[1] == '\0') {
@@ -242,7 +221,14 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 			return false;
 		}
 
-		opt->object = readlinkf((*argv)[1]);
+		/* Only absolute paths should be in mtab. */
+		ret = HX_realpath(&opt->object, (*argv)[1],
+		      HX_REALPATH_DEFAULT | HX_REALPATH_ABSOLUTE);
+		if (ret < 0) {
+			fprintf(stderr, "realpath %s: %s\n",
+			        (*argv)[1], strerror(-ret));
+			return false;
+		}
 		if (stat(opt->object, &sb) < 0) {
 			/* If it does not exist, it cannot be the container. */
 			opt->is_cont = false;
@@ -273,8 +259,20 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 		return false;
 	}
 
-	opt->container  = readlinkf((*argv)[1]);
-	opt->mountpoint = readlinkf((*argv)[2]);
+	ret = HX_realpath(&opt->container, (*argv)[1],
+	      HX_REALPATH_DEFAULT | HX_REALPATH_ABSOLUTE);
+	if (ret < 0) {
+		fprintf(stderr, "realpath %s: %s\n",
+		        (*argv)[1], strerror(-ret));
+		return false;
+	}
+	ret = HX_realpath(&opt->mountpoint, (*argv)[2],
+	      HX_REALPATH_DEFAULT | HX_REALPATH_ABSOLUTE);
+	if (ret < 0) {
+		fprintf(stderr, "realpath %s: %s\n",
+		        (*argv)[2], strerror(-ret));
+		return false;
+	}
 
 	if (stat(opt->mountpoint, &sb) < 0) {
 		fprintf(stderr, "%s: stat %s: %s\n", **argv, opt->mountpoint,
@@ -318,8 +316,7 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 		}
 	}
 
-#ifdef HAVE_LIBCRYPTSETUP
-	ret = dmc_is_luks(opt->container, opt->blkdev);
+	ret = ehd_is_luks(opt->container, opt->blkdev);
 	if (ret > 0) {
 		/* LUKS */
 		if (opt->dmcrypt_cipher != NULL)
@@ -336,12 +333,11 @@ static bool mtcr_get_mount_options(int *argc, const char ***argv,
 			return false;
 		}
 	}
-#endif
 
 	if (opt->dmcrypt_hash == NULL)
 		opt->dmcrypt_hash = "plain";
 	if (!kfpt)
-		opt->fsk_password = pmt_get_password(NULL);
+		opt->fsk_password = ehd_get_password(NULL);
 	return true;
 }
 
@@ -374,6 +370,33 @@ static hxmc_t *mtcr_slurp_file(const char *file)
 	return buf;
 }
 
+static int mtcr_decrypt_keyfile(const struct mount_options *opt,
+    hxmc_t **result)
+{
+	struct ehd_keydec_request *dp;
+	int ret;
+
+	dp = ehd_kdreq_new();
+	if (dp == NULL)
+		return -errno;
+	ret = ehd_kdreq_set(dp, EHD_KDREQ_KEYFILE, opt->fsk_file);
+	if (ret < 0)
+		goto out;
+	ret = ehd_kdreq_set(dp, EHD_KDREQ_DIGEST, opt->fsk_hash);
+	if (ret < 0)
+		goto out;
+	ret = ehd_kdreq_set(dp, EHD_KDREQ_CIPHER, opt->fsk_cipher);
+	if (ret < 0)
+		goto out;
+	ret = ehd_kdreq_set(dp, EHD_KDREQ_PASSWORD, opt->fsk_password);
+	if (ret < 0)
+		goto out;
+	ret = ehd_keydec_run(dp, result);
+ out:
+	ehd_kdreq_free(dp);
+	return ret;
+}
+
 /**
  * mtcr_mount
  *
@@ -381,75 +404,102 @@ static hxmc_t *mtcr_slurp_file(const char *file)
  */
 static int mtcr_mount(struct mount_options *opt)
 {
-	const char *mount_args[9];
+	const char *mount_args[8];
 	const char *fsck_args[4];
 	struct stat sb;
+	hxmc_t *key = NULL;
 	int ret, argk;
-	hxmc_t *key;
-	struct ehd_mount mount_info;
-	struct ehd_mtreq mount_request = {
-		.container = opt->container,
-		.fs_cipher = opt->dmcrypt_cipher,
-		.fs_hash   = opt->dmcrypt_hash,
-		.readonly  = opt->readonly,
+	struct ehd_mount_info *mount_info;
+	struct ehd_mount_request *mount_request;
+	unsigned int key_size = 0, trunc_keysize;
 
-		/* Hack for CRYPT_PLAIN. */
-		.trunc_keysize = 256 / CHAR_BIT,
-	};
+	mount_request = ehd_mtreq_new();
+	if (mount_request == NULL) {
+		fprintf(stderr, "%s\n", strerror(errno));
+		return 0;
+	}
+	ret = ehd_mtreq_set(mount_request, EHD_MTREQ_CONTAINER, opt->container);
+	if (ret < 0)
+		goto out_r;
+	ret = ehd_mtreq_set(mount_request, EHD_MTREQ_CRYPTONAME, opt->crypto_name);
+	if (ret < 0)
+		goto out_r;
+	ret = ehd_mtreq_set(mount_request, EHD_MTREQ_FS_CIPHER, opt->dmcrypt_cipher);
+	if (ret < 0)
+		goto out_r;
+	ret = ehd_mtreq_set(mount_request, EHD_MTREQ_FS_HASH, opt->dmcrypt_hash);
+	if (ret < 0)
+		goto out_r;
+	ret = ehd_mtreq_set(mount_request, EHD_MTREQ_READONLY, opt->readonly);
+	if (ret < 0)
+		goto out_r;
+	/* Hack for CRYPT_PLAIN: default to 256 */
+	trunc_keysize = 256 / CHAR_BIT;
+	ret = ehd_mtreq_set(mount_request, EHD_MTREQ_TRUNC_KEYSIZE, trunc_keysize);
+	if (ret < 0)
+		goto out_r;
 
 	if (opt->fsk_file == NULL) {
 		/* LUKS derives the key material on its own */
-		key = HXmc_meminit(opt->fsk_password,
-		      HXmc_length(opt->fsk_password));
-		if (key == NULL) {
-			fprintf(stderr, "HXmc_dup: %s\n", strerror(errno));
-			return 0;
-		}
+		ret = ehd_mtreq_set(mount_request, EHD_MTREQ_KEY_SIZE, HXmc_length(opt->fsk_password));
+		if (ret < 0)
+			goto out_r;
+		ret = ehd_mtreq_set(mount_request, EHD_MTREQ_KEY_DATA, opt->fsk_password);
+		if (ret < 0)
+			goto out_r;
 		/* Leave trunc_keysize at 0 */
 	} else if (kfpt_selected(opt->fsk_cipher)) {
 		key = mtcr_slurp_file(opt->fsk_file);
-		if (key == NULL)
-			return 0;
-		mount_request.trunc_keysize = HXmc_length(key);
-	} else {
-#ifdef HAVE_LIBCRYPTO
-		key = ehd_decrypt_key(opt->fsk_file, opt->fsk_hash,
-		      opt->fsk_cipher, opt->fsk_password);
 		if (key == NULL) {
-			fprintf(stderr, "Error while decrypting fskey\n");
-			return 0;
+			ret = -errno;
+			goto out_r;
+		}
+	} else {
+		ret = mtcr_decrypt_keyfile(opt, &key);
+		if (ret != EHD_KEYDEC_SUCCESS || key == NULL) {
+			fprintf(stderr, "Error while decrypting fskey: %s\n",
+			        ehd_keydec_strerror(ret));
+			goto out_z;
 		}
-		mount_request.trunc_keysize = HXmc_length(key);
-#else
-		fprintf(stderr, "mtcrypt was compiled without OpenSSL support\n");
-		return 0;
-#endif
 	}
 
-	mount_request.key_data = key;
-	mount_request.key_size = HXmc_length(key);
-	if (opt->trunc_keysize != 0)
-		mount_request.trunc_keysize = opt->trunc_keysize;
+	if (key != NULL) {
+		key_size = HXmc_length(key);
+		ret = ehd_mtreq_set(mount_request, EHD_MTREQ_TRUNC_KEYSIZE, key_size);
+		if (ret < 0)
+			goto out_r;
+		ret = ehd_mtreq_set(mount_request, EHD_MTREQ_KEY_SIZE, key_size);
+		if (ret < 0)
+			goto out_r;
+		ret = ehd_mtreq_set(mount_request, EHD_MTREQ_KEY_DATA, key);
+		HXmc_free(key);
+		key = NULL;
+		if (ret < 0)
+			goto out_r;
+	}
+	if (opt->trunc_keysize != 0) {
+		ret = ehd_mtreq_set(mount_request, EHD_MTREQ_TRUNC_KEYSIZE, opt->trunc_keysize);
+		if (ret < 0)
+			goto out_r;
+	}
 
-	w4rn("keysize=%u trunc_keysize=%u\n", mount_request.key_size,
-	     mount_request.trunc_keysize);
-	if ((ret = ehd_load(&mount_request, &mount_info)) < 0) {
+	w4rn("keysize=%u trunc_keysize=%u\n", key_size, trunc_keysize);
+	if ((ret = ehd_load(mount_request, &mount_info)) < 0) {
 		fprintf(stderr, "ehd_load: %s\n", strerror(errno));
-		return 0;
+		goto out_z;
 	} else if (ret == 0) {
-		return 0;
+		goto out_z;
 	}
-	HXmc_free(key);
-	if (mount_info.crypto_device == NULL) {
-		if (Debug)
+	if (mount_info->crypto_device == NULL) {
+		if (mtcr_debug)
 			fprintf(stderr, "No crypto device assigned\n");
-		ehd_unload(&mount_info);
-		ehd_mtfree(&mount_info);
-		return 0;
+		ehd_unload(mount_info);
+		ehd_mtinfo_free(mount_info);
+		goto out_z;
 	}
 
 	opt->dm_timeout *= 3;
-	while (stat(mount_info.crypto_device, &sb) < 0 && errno == ENOENT &&
+	while (stat(mount_info->crypto_device, &sb) < 0 && errno == ENOENT &&
 	    opt->dm_timeout-- > 0)
 		usleep(333333);
 
@@ -457,7 +507,7 @@ static int mtcr_mount(struct mount_options *opt)
 		argk = 0;
 		fsck_args[argk++] = "fsck";
 		fsck_args[argk++] = "-p";
-		fsck_args[argk++] = mount_info.crypto_device;
+		fsck_args[argk++] = mount_info->crypto_device;
 		fsck_args[argk] = NULL;
 		assert(argk < ARRAY_SIZE(fsck_args));
 
@@ -473,17 +523,15 @@ static int mtcr_mount(struct mount_options *opt)
 			fprintf(stderr, "Automatic fsck failed, manual "
 			        "intervention required, run_sync status %d\n",
 			        ret);
-			ehd_unload(&mount_info);
-			ehd_mtfree(&mount_info);
-			return false;
+			ehd_unload(mount_info);
+			ehd_mtinfo_free(mount_info);
+			goto out_z;
 		}
 	}
 
+	/* candidate for replacement by some libmount calls, I guess. */
 	argk = 0;
 	mount_args[argk++] = "mount";
-#ifdef __linux__
-	mount_args[argk++] = "-n";
-#endif
 	if (opt->fstype != NULL) {
 		mount_args[argk++] = "-t";
 		mount_args[argk++] = opt->fstype;
@@ -492,7 +540,7 @@ static int mtcr_mount(struct mount_options *opt)
 		mount_args[argk++] = "-o";
 		mount_args[argk++] = opt->extra_opts;
 	}
-	mount_args[argk++] = mount_info.crypto_device;
+	mount_args[argk++] = mount_info->crypto_device;
 	mount_args[argk++] = opt->mountpoint;
 	mount_args[argk] = NULL;
 
@@ -500,23 +548,29 @@ static int mtcr_mount(struct mount_options *opt)
 	arglist_llog(mount_args);
 	if ((ret = HXproc_run_sync(mount_args, HXPROC_VERBOSE)) != 0) {
 		fprintf(stderr, "mount failed with run_sync status %d\n", ret);
-		ehd_unload(&mount_info);
+		ehd_unload(mount_info);
 		ret = 0;
 	} else if ((ret = pmt_cmtab_add(opt->mountpoint,
-	    mount_info.container, mount_info.loop_device,
-	    mount_info.crypto_device)) <= 0) {
+	    mount_info->container, mount_info->loop_device,
+	    mount_info->crypto_device)) <= 0) {
 		fprintf(stderr, "pmt_cmtab_add: %s\n", strerror(errno));
 		/* ignore error on cmtab - let user have his crypto */
 	} else if (opt->no_update) {
 		/* awesome logic */;
 	} else {
-		pmt_smtab_add(mount_info.container, opt->mountpoint,
+		pmt_smtab_add(mount_info->container, opt->mountpoint,
 			"crypt", (opt->extra_opts != NULL) ?
 			opt->extra_opts : "defaults");
 	}
 
-	ehd_mtfree(&mount_info);
+	ehd_mtinfo_free(mount_info);
 	return ret;
+
+ out_r:
+	fprintf(stderr, "ehd_mtreq_set: %s\n", strerror(-ret));
+ out_z:
+	HXmc_free(key);
+	return 0;
 }
 
 static bool mtcr_get_umount_options(int *argc, const char ***argv,
@@ -530,22 +584,33 @@ static bool mtcr_get_umount_options(int *argc, const char ***argv,
 		 .help = "Do not update /etc/mtab"},
 		{.sh = 'r', .type = HXTYPE_NONE, .ptr = &opt->ro_fallback,
 		 .help = "(Option ignored)"},
-		{.sh = 'v', .type = HXTYPE_NONE, .ptr = &Debug,
+		{.sh = 'v', .type = HXTYPE_NONE, .ptr = &mtcr_debug,
 		 .help = "Be verbose - enable debugging"},
 		HXOPT_AUTOHELP,
 		HXOPT_TABLEEND,
 	};
+	int ret;
 
-	if (HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR) <= 0)
+	if (HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR) !=
+	    HXOPT_ERR_SUCCESS)
 		return false;
 
+	if (mtcr_debug)
+		ehd_logctl(EHD_LOGFT_DEBUG, EHD_LOG_SET);
+
 	if (*argc < 2 || *(*argv)[1] == '\0') {
 		fprintf(stderr, "%s: You need to specify the container "
 		        "or its mountpoint\n", **argv);
 		return false;
 	}
 
-	opt->object = readlinkf((*argv)[1]);
+	ret = HX_realpath(&opt->object, (*argv)[1],
+	      HX_REALPATH_DEFAULT | HX_REALPATH_ABSOLUTE);
+	if (ret < 0) {
+		fprintf(stderr, "realpath %s: %s\n",
+		        (*argv)[1], strerror(-ret));
+		return false;
+	}
 	if (stat(opt->object, &sb) < 0) {
 		/* If it does not exist, it cannot be the container. */
 		opt->is_cont = false;
@@ -570,12 +635,12 @@ static bool mtcr_get_umount_options(int *argc, const char ***argv,
  */
 static int mtcr_remount(struct mount_options *opt)
 {
-	const char *rmt_args[6];
+	const char *rmt_args[5];
 	int ret, argk = 0;
-	char *mntpt;
+	char *mntpt, *cont;
 
 	ret = pmt_cmtab_get(opt->object, opt->is_cont ?
-	      CMTABF_CONTAINER : CMTABF_MOUNTPOINT, &mntpt, NULL, NULL, NULL);
+	      CMTABF_CONTAINER : CMTABF_MOUNTPOINT, &mntpt, &cont, NULL, NULL);
 	if (ret == 0) {
 		fprintf(stderr, "Nothing found that could be remounted.\n");
 		return 1;
@@ -584,10 +649,9 @@ static int mtcr_remount(struct mount_options *opt)
 		return ret;
 	}
 
+	if (!opt->no_update)
+		pmt_smtab_remove(mntpt, SMTABF_MOUNTPOINT);
 	rmt_args[argk++] = "mount";
-#ifdef __linux__
-	rmt_args[argk++] = "-i";
-#endif
 	rmt_args[argk++] = "-o";
 	rmt_args[argk++] = opt->extra_opts;
 	rmt_args[argk++] = mntpt;
@@ -599,10 +663,36 @@ static int mtcr_remount(struct mount_options *opt)
 		fprintf(stderr, "remount %s failed with run_sync status %d\n",
 		        opt->object, ret);
 
+	if (!opt->no_update)
+		pmt_smtab_add(cont, mntpt, "crypt", (opt->extra_opts != NULL) ?
+			opt->extra_opts : "defaults");
 	free(mntpt);
+	free(cont);
 	return ret;
 }
 
+/**
+ */
+static void mtcr_log_contents(const char *file)
+{
+	hxmc_t *ln = NULL;
+	FILE *fp;
+
+	if (file == NULL)
+		return;
+	w4rn("Dumping contents of %s\n", file);
+	if ((fp = fopen(file, "r")) == NULL) {
+		w4rn("Failed to open %s: %s\n", file, strerror(errno));
+		return;
+	}
+	while (HX_getl(&ln, fp) != NULL) {
+		HX_chomp(ln);
+		w4rn("%s\n", ln);
+	}
+	HXmc_free(ln);
+	fclose(fp);
+}
+
 /**
  * mtcr_umount - unloads the EHD from mountpoint
  *
@@ -610,9 +700,9 @@ static int mtcr_remount(struct mount_options *opt)
  */
 static int mtcr_umount(struct umount_options *opt)
 {
-	const char *umount_args[5];
+	const char *umount_args[3];
 	int final_ret, ret, argk = 0;
-	struct ehd_mount mount_info;
+	struct ehd_mount_info mount_info;
 	char *mountpoint = NULL;
 
 	memset(&mount_info, 0, sizeof(mount_info));
@@ -623,16 +713,26 @@ static int mtcr_umount(struct umount_options *opt)
 		fprintf(stderr, "pmt_cmtab_get: %s\n", strerror(-ret));
 		return 0;
 	} else if (ret == 0) {
-		fprintf(stderr, "%s is not mounted (according to cmtab)\n",
-		        opt->object);
+		fprintf(stderr, "No vfsmount found while searching for \"%s\" "
+		        "as a container file, or as a mountpoint. (According "
+		        "to the intersection of cmtab (%s) with smtabs)\n",
+		        opt->object, pmt_cmtab_path());
+		mtcr_log_contents(pmt_cmtab_path());
+		mtcr_log_contents(pmt_smtab_path());
+		mtcr_log_contents(pmt_kmtab_path());
 		return 1;
+	} else {
+		if (ret & PMT_BY_CONTAINER)
+			w4rn("Found container in smtab\n");
+		if (ret & PMT_BY_CRYPTODEV)
+			w4rn("Found crypto device in smtab\n");
 	}
 
+	if (!opt->no_update)
+		pmt_smtab_remove(mountpoint, SMTABF_MOUNTPOINT);
+	pmt_cmtab_remove(mountpoint);
+
 	umount_args[argk++] = "umount";
-#ifdef __linux__
-	/* Always pass in -n, as we manually edit /etc/mtab */
-	umount_args[argk++] = "-ni";
-#endif
 	umount_args[argk++] = mountpoint;
 	umount_args[argk]   = NULL;
 
@@ -647,29 +747,15 @@ static int mtcr_umount(struct umount_options *opt)
 		fprintf(stderr, "ehd_unload: %s\n", strerror(-ret));
 		final_ret = 0;
 	} else {
-		if (!opt->no_update)
-			pmt_smtab_remove(mountpoint, SMTABF_MOUNTPOINT);
-		pmt_cmtab_remove(mountpoint, CMTABF_MOUNTPOINT);
 		final_ret = 1;
 	}
 
 	return final_ret;
 }
 
-int main(int argc, const char **argv)
+static int main2(int argc, const char **argv)
 {
-	int ret;
-
-	ret = HX_init();
-	if (ret <= 0) {
-		fprintf(stderr, "HX_init: %s\n", strerror(errno));
-		abort();
-	}
-
-	Debug = false;
-	pmtlog_path[PMTLOG_ERR][PMTLOG_STDERR] = true;
-	pmtlog_prefix = HX_basename(*argv);
-
+	ehd_logctl(EHD_LOGFT_NOSYSLOG, EHD_LOG_SET);
 	setenv("PATH", PMT_DFL_PATH, true);
 #ifdef HAVE_LIBCRYPTO
 	OpenSSL_add_all_ciphers();
@@ -706,3 +792,32 @@ int main(int argc, const char **argv)
 
 	return EXIT_FAILURE;
 }
+
+int main(int argc, const char **argv)
+{
+	struct stat sb;
+	int ret;
+
+	if (stat("/etc/mtab", &sb) == 0 && (sb.st_mode & S_IWUGO) == 0)
+		fprintf(stderr, "NOTE: mount.crypt does not support utab "
+		        "(systems with no mtab or read-only mtab) yet. This "
+		        "means that you will temporarily need to call "
+		        "umount.crypt(8) rather than umount(8) to get crypto "
+		        "volumes unmounted.\n");
+
+	ret = HX_init();
+	if (ret <= 0) {
+		fprintf(stderr, "HX_init: %s\n", strerror(errno));
+		abort();
+	}
+	ret = cryptmount_init();
+	if (ret <= 0) {
+		fprintf(stderr, "cryptmount_init: %s\n", strerror(errno));
+		abort();
+	}
+
+	ret = main2(argc, argv);
+	cryptmount_exit();
+	HX_exit();
+	return ret;
+}
