@@ -15,12 +15,6 @@
 #	define EXPORT_SYMBOL
 #endif
 
-#ifndef S_IRXG
-#	define S_IRXG (S_IRGRP | S_IXGRP)
-#endif
-#ifndef S_IRXO
-#	define S_IRXO (S_IROTH | S_IXOTH)
-#endif
 #define sizeof_z(x) (sizeof(x) - 1)
 
 /*
@@ -28,17 +22,17 @@
  * so just provide our own.
  */
 #define PMT_DFL_PATH \
-	"/sbin:/bin:/usr/sbin:/usr/bin:/usr/local/sbin:/usr/local/bin"
-#define PMT_DFL_DMCRYPT_CIPHER "aes-cbc-essiv:sha256"
-#define PMT_DFL_FSK_CIPHER     "aes-256-cbc"
-#define PMT_DFL_FSK_HASH       "sha1"
+	"/usr/local/libexec/hxtools:/usr/local/lib/hxtools:" \
+	"/usr/local/sbin:/usr/local/bin:" \
+	"/usr/libexec/hxtools:/usr/lib/hxtools:" \
+	"/usr/sbin:/usr/bin:/sbin:/bin"
 
 /* Note that you will also need to change PMPREFIX in pmvarrun.c then! */
 #define l0g(fmt, ...) \
-	misc_log(("%s(%s:%u): " fmt), pmtlog_prefix, HX_basename(__FILE__), \
+	ehd_err(("(%s:%u): " fmt), HX_basename(__FILE__), \
 	__LINE__, ## __VA_ARGS__)
 #define w4rn(fmt, ...) \
-	misc_warn(("%s(%s:%u): " fmt), pmtlog_prefix, HX_basename(__FILE__), \
+	ehd_dbg(("(%s:%u): " fmt), HX_basename(__FILE__), \
 	__LINE__, ## __VA_ARGS__)
 
 struct HXdeque;
@@ -46,13 +40,6 @@ struct HXformatmap;
 struct HXproc;
 struct loop_info64;
 
-enum auth_type {
-	GET_PASS,
-	USE_FIRST_PASS,
-	TRY_FIRST_PASS,
-	SOFT_TRY_PASS,
-};
-
 enum command_type {
 	CMD_SMBMOUNT,
 	CMD_SMBUMOUNT,
@@ -70,22 +57,16 @@ enum command_type {
 	CMD_FSCK,
 	CMD_PMVARRUN,
 	CMD_FD0SSH,
+	CMD_OFL,
 	_CMD_MAX,
 	CMD_NONE,
 };
 
-enum {
-	/* src */
-	PMTLOG_ERR = 0,
-	PMTLOG_DBG,
-	PMTLOG_SRCMAX,
-
-	/* dst */
-	PMTLOG_SYSLOG = 0,
-	PMTLOG_STDERR,
-	PMTLOG_DSTMAX,
-};
-
+/**
+ * @server:	server name, if any
+ * @volume:	path relative to server, or full path in case @server is empty
+ * @combopath:	concatenation of @server and @volume dependent upon @fstype
+ */
 struct vol {
 	struct HXlist_head list;
 	enum command_type type;
@@ -98,7 +79,7 @@ struct vol {
 	/* was handed off to mount_op() */
 	bool mnt_processed;
 	const char *user;
-	char *fstype, *server, *volume, *mountpoint, *cipher;
+	char *fstype, *server, *volume, *combopath, *mountpoint, *cipher;
 	char *fs_key_cipher, *fs_key_hash, *fs_key_path;
 	/* May be NULL if no options */
 	struct HXclist_head options;
@@ -136,47 +117,6 @@ struct kvp {
 	struct HXlist_head list;
 };
 
-/**
- * struct ehd_request - mapping and mount request for EHD
- * @mountpoint:	where to mount EHD
- * @fs_cipher:	cipher used for filesystem (cryptsetup name)
- * @fs_hash:	hash used for filesystem (cryptsetup name)
- * @container:	path to disk image
- * @key_data:	key material
- * @key_size:	size of key data, in bytes
- * @trunc_keysize:	extra cryptsetup instruction for truncation (in bytes)
- * @readonly:	create readonly mount?
- */
-struct ehd_mtreq {
-	const char *mountpoint;
-	const char *fs_cipher, *fs_hash, *container;
-	const void *key_data;
-	unsigned int key_size, trunc_keysize;
-	bool readonly;
-};
-
-/**
- * struct ehd_mount - EHD mount info
- * @container:		path to disk image
- * @lower_device:	link to either @container if a block device,
- * 			otherwise points to @loop_device.
- * @loop_device:	loop device that was created, if any
- * @crypto_name:	crypto device that was created (basename only)
- * @crypto_device:	full path to the crypto device
- */
-struct ehd_mount {
-	char *container;
-	const char *lower_device;
-	char *loop_device;
-	hxmc_t *crypto_name;
-	hxmc_t *crypto_device;
-};
-
-struct ehd_crypto_ops {
-	int (*load)(const struct ehd_mtreq *, struct ehd_mount *);
-	int (*unload)(const struct ehd_mount *);
-};
-
 typedef int (mount_op_fn_t)(const struct config *, struct vol *,
 	struct HXformat_map *, const char *);
 
@@ -198,39 +138,9 @@ static inline const char *znul(const char *s)
 }
 
 /*
- *	CRYPTO.C
- */
-extern int ehd_load(const struct ehd_mtreq *, struct ehd_mount *);
-extern int ehd_unload(const struct ehd_mount *);
-extern void ehd_mtfree(struct ehd_mount *);
-extern hxmc_t *ehd_decrypt_key(const char *, const char *, const char *,
-	hxmc_t *);
-extern unsigned int cipher_digest_security(const char *);
-extern hxmc_t *pmt_get_password(const char *);
-
-/*
- *	CRYPTO-*.C
- */
-extern const struct ehd_crypto_ops ehd_cgd_ops;
-extern const struct ehd_crypto_ops ehd_dmcrypt_ops;
-
-extern int dmc_is_luks(const char *, bool);
-
-/*
- *	LOOP.C
+ *	BDEV.C
  */
 extern size_t pmt_block_getsize64(const char *);
-extern int pmt_loop_setup(const char *, char **, bool);
-extern int pmt_loop_release(const char *);
-
-/*
- *	LOG.C
- */
-extern const char *pmtlog_prefix;
-extern bool pmtlog_path[PMTLOG_SRCMAX][PMTLOG_DSTMAX];
-
-extern int misc_log(const char *, ...);
-extern int misc_warn(const char *, ...);
 
 /*
  *	MISC.C
@@ -248,11 +158,10 @@ extern hxmc_t *kvplist_to_str(const struct HXclist_head *);
 extern void misc_add_ntdom(struct HXformat_map *, const char *);
 extern void misc_dump_id(const char *);
 extern int pmt_fileop_exists(const char *);
+extern int pmt_fileop_isreg(const char *);
 extern int pmt_fileop_owns(const char *, const char *);
 extern char *relookup_user(const char *);
 extern long str_to_long(const char *);
-extern void *xmalloc(size_t);
-extern void *xrealloc(void *, size_t);
 extern char *xstrdup(const char *);
 
 /*
@@ -273,6 +182,11 @@ enum cmtab_field {
 	__CMTABF_MAX,
 };
 
+enum {
+	PMT_BY_CONTAINER = 1 << 0,
+	PMT_BY_CRYPTODEV = 1 << 1,
+};
+
 extern int pmt_smtab_add(const char *, const char *,
 	const char *, const char *);
 extern int pmt_smtab_remove(const char *, enum smtab_field);
@@ -282,8 +196,11 @@ extern int pmt_cmtab_add(const char *, const char *,
 	const char *, const char *);
 extern int pmt_cmtab_get(const char *, enum cmtab_field,
 	char **, char **, char **, char **);
-extern int pmt_cmtab_remove(const char *, enum cmtab_field);
+extern int pmt_cmtab_remove(const char *);
 extern int pmt_cmtab_mounted(const char *, const char *);
+extern const char *pmt_cmtab_path(void);
+extern const char *pmt_smtab_path(void);
+extern const char *pmt_kmtab_path(void);
 
 /*
  *	MOUNT.C
@@ -308,11 +225,6 @@ extern bool ofl(const char *, unsigned int);
 /*
  *	PAM_MOUNT.C
  */
-#ifndef HAVE_VISIBILITY_HIDDEN
-	/* Workaround Xserver issue */
-#	define Debug pmt_debug
-#endif
-extern unsigned int Debug;
 extern struct config Config;
 
 /*
