@@ -1,5 +1,5 @@
 /*
- *	Copyright © Jan Engelhardt, 2006 - 2009
+ *	Copyright © Jan Engelhardt, 2006 - 2011
  *
  *	This file is part of pam_mount; you can redistribute it and/or
  *	modify it under the terms of the GNU Lesser General Public License
@@ -21,6 +21,7 @@
 #endif
 #include <libHX.h>
 #include <libHX/libxml_helper.h>
+#include "libcryptmount.h"
 #include "pam_mount.h"
 
 /* Definitions */
@@ -97,8 +98,8 @@ static bool expand_home(const char *user, char **path_pptr)
 		return false;
 	}
 	size = strlen(pe->pw_dir) + strlen(path) + 1;
-	if ((buf = xmalloc(size)) == NULL) {
-		l0g("%s\n", strerror(errno));
+	if ((buf = malloc(size)) == NULL) {
+		l0g("%s: malloc %zu: %s\n", __func__, size, strerror(errno));
 		return NULL;
 	}
 	snprintf(buf, size, "%s%s", pe->pw_dir, path + 1);
@@ -115,9 +116,6 @@ static bool expand_home(const char *user, char **path_pptr)
  *
  * Substitutes all occurrences of %(USER) by the username. Returns NULL on
  * failure, otherwise @dest.
- *
- * (This should probably be done by the fmt_ptrn stuff, but is not at the
- * moment due to to-XML transition period.)
  */
 static bool expand_user(const char *user, char **dest_pptr,
     const struct HXformat_map *vinfo)
@@ -126,7 +124,7 @@ static bool expand_user(const char *user, char **dest_pptr,
 
 	if (*dest_pptr == NULL)
 		return true;
-	HXformat2_aprintf(vinfo, &tmp, *dest_pptr);
+	HXformat_aprintf(vinfo, &tmp, *dest_pptr);
 	*dest_pptr = xstrdup(tmp);
 	HXmc_free(tmp);
 	return true;
@@ -156,6 +154,13 @@ bool expandconfig(const struct config *config)
 		goto rfalse;
 	}
 
+	/*
+	 * Because user's volumes will also be mounted with UID 0,
+	 * deactivate shell expansion for now.
+	 */
+	if (config->level == CONTEXT_GLOBAL)
+		HXformat_add(vinfo, "/libhx/exec", NULL, HXFORMAT_IMMED);
+
 	HXformat_add(vinfo, "USER", u, HXTYPE_STRING);
 	HXformat_add(vinfo, "USERUID", reinterpret_cast(void *,
 		static_cast(long, pe->pw_uid)), HXTYPE_UINT | HXFORMAT_IMMED);
@@ -182,7 +187,7 @@ bool expandconfig(const struct config *config)
 		/*
 		 * Substitution on the key seems odd indeed, and was not
 		 * originally intended. But it should be done, since the
-		 * option separator is notalways ",", e.g. with AUFS.
+		 * option separator is not always ",", e.g. with AUFS.
 		 * Least Surprise says: do the expansion on the key
 		 * nevertheless.
 		 */
@@ -205,6 +210,7 @@ static void volume_free(struct vol *vol)
 	free(vol->fstype);
 	free(vol->server);
 	free(vol->volume);
+	HXmc_free(vol->combopath);
 	free(vol->cipher);
 	free(vol->fs_key_cipher);
 	free(vol->fs_key_hash);
@@ -276,9 +282,11 @@ static bool str_to_optkv(struct HXclist_head *optlist, char *str)
 		return true;
 
 	while ((ptr = HX_strsep(&str, ",")) != NULL) {
-		kvp = xmalloc(sizeof(struct kvp));
-		if (kvp == NULL)
+		kvp = malloc(sizeof(struct kvp));
+		if (kvp == NULL) {
+			l0g("%s: malloc: %s\n", __func__, strerror(errno));
 			return false;
+		}
 		HXlist_init(&kvp->list);
 		value = strchr(ptr, '=');
 		if (value != NULL) {
@@ -647,12 +655,13 @@ static const char *rc_debug(xmlNode *node, struct config *config,
     unsigned int cmd)
 {
 	char *s;
-	if ((s = xml_getprop(node, "enable")) != NULL)
-		Debug = config->debug = strtoul(s, NULL, 0);
-
-		pmtlog_path[PMTLOG_DBG][PMTLOG_SYSLOG] =
-		pmtlog_path[PMTLOG_DBG][PMTLOG_STDERR] =
-			Debug;
+	if ((s = xml_getprop(node, "enable")) != NULL) {
+		if (config->debug)
+			ehd_logctl(EHD_LOGFT_DEBUG, EHD_LOG_UNSET);
+		config->debug = strtoul(s, NULL, 0);
+		if (config->debug)
+			ehd_logctl(EHD_LOGFT_DEBUG, EHD_LOG_SET);
+	}
 	free(s);
 	return NULL;
 }
@@ -1255,14 +1264,11 @@ static const char *rc_volume(xmlNode *node, struct config *config,
 		vpt->fstype = xstrdup("auto");
 	}
 
-	if ((tmp = xml_getprop(node, "noroot")) != NULL) {
+	if ((tmp = xml_getprop(node, "noroot")) != NULL)
 		vpt->noroot = parse_bool_f(tmp);
-	} else if (vpt->fstype != NULL) {
+	else if (vpt->fstype != NULL)
 		/* Figure out whether we want to act as user. */
-		vpt->noroot =
-			strcmp(vpt->fstype, "fuse") == 0 ||
-			strcmp(vpt->fstype, "encfs13") == 0;
-	}
+		vpt->noroot = strcmp(vpt->fstype, "fuse") == 0;
 
 	/* Source location */
 	if ((tmp = xml_getprop(node, "server")) != NULL) {
@@ -1350,24 +1356,24 @@ static const char *rc_volume(xmlNode *node, struct config *config,
 
 //-----------------------------------------------------------------------------
 static const struct pmt_command default_command[] = {
-	{CMD_SMBMOUNT,   "smbfs", {"smbmount", "//%(SERVER)/%(VOLUME)", "%(MNTPT)", "-o", "username=%(USER),uid=%(USERUID),gid=%(USERGID)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
+	{CMD_SMBMOUNT,   "smbfs", {"smbmount", "%(COMBOPATH)", "%(MNTPT)", "-o", "username=%(USER),uid=%(USERUID),gid=%(USERGID)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
 	{CMD_SMBUMOUNT,  "smbfs", {"smbumount", "%(MNTPT)", NULL}},
-	{CMD_CIFSMOUNT,   "cifs", {"mount", "-t", "cifs", "//%(SERVER)/%(VOLUME)", "%(MNTPT)", "-o", "user=%(USER),uid=%(USERUID),gid=%(USERGID)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
-	{CMD_NCPMOUNT,   "ncpfs", {"ncpmount", "%(SERVER)/%(USER)", "%(MNTPT)", "-o", "pass-fd=0,volume=%(VOLUME)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
+	{CMD_CIFSMOUNT,   "cifs", {"mount", "-t", "cifs", "%(COMBOPATH)", "%(MNTPT)", "-o", "user=%(USER),uid=%(USERUID),gid=%(USERGID)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
+	{CMD_NCPMOUNT,   "ncpfs", {"ncpmount", "%(COMBOPATH)", "%(MNTPT)", "-o", "pass-fd=0,volume=%(VOLUME)%(if %(OPTIONS),\",%(OPTIONS)\")", NULL}},
 	{CMD_NCPUMOUNT,  "ncpfs", {"ncpumount", "%(MNTPT)", NULL}},
-	{CMD_FUSEMOUNT,   "fuse", {"mount.fuse", "%(VOLUME)", "%(MNTPT)", "%(if %(OPTIONS),-o%(OPTIONS))", NULL}},
+	{CMD_FUSEMOUNT,   "fuse", {"mount.fuse", "%(VOLUME)", "%(MNTPT)", "%(if %(OPTIONS),-o)", "%(OPTIONS)", NULL}},
 	{CMD_FUSEUMOUNT, "fuse",  {"fusermount", "-u", "%(MNTPT)", NULL}},
 	/* Do not use LCLMOUNT to avoid calling fsck */
-	{CMD_NFSMOUNT,    "nfs",   {"mount", "%(if %(OPTIONS),-o%(OPTIONS))", "-t%(FSTYPE)", "%(SERVER):%(VOLUME)", "%(MNTPT)", NULL}},
+	{CMD_NFSMOUNT,    "nfs",   {"mount", "%(if %(OPTIONS),-o%(OPTIONS))", "-t%(FSTYPE)", "%(COMBOPATH)", "%(MNTPT)", NULL}},
 	{CMD_NFSMOUNT,    "nfs4"},
 	{CMD_LCLMOUNT,    NULL,    {"mount", "-p0", "%(if %(OPTIONS),-o%(OPTIONS))", "-t%(FSTYPE)", "%(VOLUME)", "%(MNTPT)", NULL}},
 	{CMD_CRYPTMOUNT,  "crypt", {"mount", "-t", "crypt", "%(if %(CIPHER),-ocipher=%(CIPHER))", "%(if %(FSKEYCIPHER),-ofsk_cipher=%(FSKEYCIPHER))", "%(if %(FSKEYHASH),-ofsk_hash=%(FSKEYHASH))", "%(if %(FSKEYPATH),-okeyfile=%(FSKEYPATH))", "%(if %(OPTIONS),-o%(OPTIONS))", "%(VOLUME)", "%(MNTPT)", NULL}},
-	{CMD_CRYPTUMOUNT, "crypt", {"umount", "%(MNTPT)", NULL}},
+	{CMD_CRYPTUMOUNT, "crypt", {"umount.crypt", "%(MNTPT)", NULL}},
 	{CMD_UMOUNT,     NULL,     {"umount", "%(MNTPT)", NULL}},
 	{CMD_FSCK,       NULL,     {"fsck", "-p", "%(FSCKTARGET)", NULL}},
 	{CMD_PMVARRUN,   NULL,     {"pmvarrun", "-u", "%(USER)", "-o", "%(OPERATION)", NULL}},
 	{CMD_FD0SSH,      NULL,    {"pmt-fd0ssh", NULL}},
-	{CMD_OFL,        NULL,     {"pmt-ofl", "-k%(SIGNAL)", "%(MNTPT)", NULL}},
+	{CMD_OFL,        NULL,     {"ofl", "-k%(SIGNAL)", "%(MNTPT)", NULL}},
 	{-1},
 };
 
